@@ -165,9 +165,9 @@ def init_trx_retweet_params(
 
     if not without_images:
         images.extend(imgs + refer_imgs)
-    if not without_quote_text:
+    if not without_quote_text and refer_text:
         lines.extend(["：", _quote_str(refer_text)])
-    else:
+    elif refer_text:
         lines.append("。")
 
     content = f"{nickname} {_dt} " + "\n".join(lines)
